/*
 * Copyright (C) 2007 Sascha Sommer
 *
 * This file is part of xine, a free video player.
 *
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * helper functions to query the internet for additional plugins
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <unistd.h>
#include <sys/wait.h>

#include "install_plugins_helper.h"

#define PROCNAME_LEN  100

static const char *
get_helper(xine_stream_t *stream)
{
  const char *helper;

  helper = getenv("XINE_INSTALL_PLUGINS_HELPER");
  if (helper == NULL)
    helper = XINE_INSTALL_PLUGINS_HELPER;

  xine_log (stream->xine, XINE_LOG_MSG,
                    _("Using plugin install helper '%s'"), helper);

  return helper;
}

static void
get_procname(char* procname, size_t len)
{
  char name[100];
  FILE* fp;
  size_t pos = 0;
  snprintf(name, sizeof(name), "/proc/%u/cmdline", getpid());

  fp = fopen(name,"rb");
  if(fp){
    while(fp && !feof(fp) && pos < sizeof(name)-1){
      procname[pos] = fgetc(fp);
      if(procname[pos] == ' ') /* ignore arguments */
        break;
      if(procname[pos] == '/') /* ignore the path to the executable */
        pos = 0;
      else
        ++pos;
    }
    fclose(fp);
  }
  procname[pos] = '\0';
}


void _x_install_plugins_helper(xine_stream_t* stream,char* plugin_type, uint32_t id, char* plugin_desc)
{
  xine_cfg_entry_t cfgentry;
  char* helper = get_helper(stream);
  FILE* fp;

  if(helper && xine_config_lookup_entry(stream->xine, "media.plugins_helper", &cfgentry) && cfgentry.num_value
       && (fp = fopen(helper,"rb") )) {
        char procname[PROCNAME_LEN];
//	char* procname = getenv("_"); /* might deliver /opt/kde3/bin/start_kdeinit_wrapper etc... */
  	pid_t pid;
        fclose(fp);
        get_procname(procname,PROCNAME_LEN);
        pid  = fork();
        if(pid == 0) {
	  size_t len = 5 + strlen(XINE_VERSION) + 1 + strlen(procname) + 1 + strlen(plugin_desc) + 1 + strlen(plugin_type) + 1 + 100 + 1;
	  char* str = calloc(1,len + 1);
	  if(str){
            snprintf(str,len,"xine|%s|%s|%s|%s=%u",XINE_VERSION,procname,plugin_desc,plugin_type,id);
            if(execl(helper,"xine-install-plugins-helper",str, NULL) == -1)
              xine_log(stream->xine, XINE_LOG_MSG,
                    _("Couldn't start plugins_helper"));
	    free(str);
          }
        }else if(pid < 0){
          xine_log(stream->xine, XINE_LOG_MSG,
                    _("Couldn't fork"));
	}else{
          waitpid(pid, NULL, 0);
        }
  }
}

